@@ -2,8 +2,12 @@
 """API models for working with device and user assets."""
 import re
 
-from ...constants import (AGG_ADAPTER_ALTS, AGG_ADAPTER_NAME, GET_SCHEMA_KEYS,
-                          GET_SCHEMAS_KEYS)
+from ...constants import (
+    AGG_ADAPTER_ALTS,
+    AGG_ADAPTER_NAME,
+    GET_SCHEMA_KEYS,
+    GET_SCHEMAS_KEYS,
+)
 from ...exceptions import ApiError, NotFoundError
 from ...tools import listify, split_str, strip_right
 from ..mixins import ChildMixins
