@@ -4,33 +4,33 @@ from ...context import CONTEXT_SETTINGS, click
 from ...options import AUTH, add_options
 from .grp_common import EXPORT, handle_export
 
-USER_NAME = click.option(
-    "--name",
-    "-n",
-    "name",
-    help="Name of user to get",
-    default=None,
-    required=False,
-    show_envvar=True,
-    show_default=True,
-)
+# USER_NAME = click.option(
+#     "--name",
+#     "-n",
+#     "name",
+#     help="Name of user to get",
+#     default=None,
+#     required=False,
+#     show_envvar=True,
+#     show_default=True,
+# )
 
 
 OPTIONS = [
     *AUTH,
     EXPORT,
-    USER_NAME,
+    # USER_NAME,
 ]
 
 
 @click.command(name="get", context_settings=CONTEXT_SETTINGS)
 @add_options(OPTIONS)
 @click.pass_context
-def cmd(ctx, url, key, secret, name, **kwargs):
-    """Get all users or a specific user."""
+def cmd(ctx, url, key, secret, **kwargs):
+    """Get all users."""
     client = ctx.obj.start_client(url=url, key=key, secret=secret)
 
     with ctx.obj.exc_wrap(wraperror=ctx.obj.wraperror):
-        data = client.system.users.get(name=name)
+        data = client.system.users.get()
 
     handle_export(ctx=ctx, data=data, **kwargs)
