@@ -15,8 +15,17 @@ import dateutil.relativedelta
 import dateutil.tz
 
 from . import __package__ as PACKAGE_ROOT
-from .constants import (ERROR_ARGS, ERROR_TMPL, NO, OK_ARGS, OK_TMPL, SIMPLE,
-                        WARN_ARGS, WARN_TMPL, YES)
+from .constants import (
+    ERROR_ARGS,
+    ERROR_TMPL,
+    NO,
+    OK_ARGS,
+    OK_TMPL,
+    SIMPLE,
+    WARN_ARGS,
+    WARN_TMPL,
+    YES,
+)
 from .exceptions import ApiError, ToolsError
 
 LOG = logging.getLogger(PACKAGE_ROOT).getChild("tools")
@@ -637,3 +646,14 @@ def sysinfo():
         attr = attr.replace("_", " ").title()
         info[attr] = value
     return info
+
+
+def calc_percent(part, whole):
+    """Pass."""
+    if 0 in [part, whole]:
+        value = 0.00
+    elif part > whole:
+        value = 100.00
+    else:
+        value = 100 * (part / whole)
+    return value
